# All detected features are included in "KERNEL_FEATURE_DETECTION"

# KERNEL_HAS_IBLKSIZE Detection [START]
#
# Find out whether the kernel supports inode->i_blksize.
# Note: i_blksize was removed in newer 2.6 kernels, but some distros have
#       patches that add it again.
# Note: Test just greps for i_blksize (so it might result in false positives,
#       but that is unlikely).
KERNEL_HAS_IBLKSIZE = $(shell \
	if grep i_blksize ${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_IBLKSIZE" ; \
	fi)
#
# KERNEL_HAS_IBLKSIZE Detection [END]

# KERNEL_HAS_FLUSH_OWNER Detection [START]
#
# Find out whether the kernel vfs flush has fl_owner_t parameter.
# Note: fl_owner_t parameter was added in vanilla 2.6.18, but SLES 10 adds it
#       to its 2.6.16 kernels.
KERNEL_HAS_FLUSH_OWNER = $(shell \
	if grep "int (\*flush) (struct file \*, fl_owner_t id);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_FLUSH_OWNER" ; \
	fi)
#
# KERNEL_HAS_FLUSH_OWNER Detection [END]

# KERNEL_HAS_STATFS_DENTRY Detection [START]
#
# Find out whether the kernel vfs statfs has a dentry parameter.
# Note: dentry parameter was added in vanilla 2.6.18, but SLES 10 adds it
#       to its 2.6.16 kernels.
KERNEL_HAS_STATFS_DENTRY = $(shell \
	if grep "int (\*statfs) (struct dentry \*, struct kstatfs \*);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_STATFS_DENTRY" ; \
	fi)
#
# KERNEL_HAS_STATFS_DENTRY Detection [END]

# KERNEL_HAS_DROP_NLINK Detection [START]
#
# Find out whether the kernel has a drop_nlink function.
# Note: Was added in vanilla 2.6.19, but RedHat adds it to 2.6.18.
KERNEL_HAS_DROP_NLINK = $(shell \
	if grep "drop_nlink(struct inode \*inode)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_DROP_NLINK" ; \
	fi)
#
# KERNEL_HAS_DROP_NLINK Detection [END]

# KERNEL_HAS_CLEAR_NLINK Detection [START]
#
# Find out whether the kernel has a clear_nlink function.
# Note: Was added in vanilla 2.6.19, but RedHat adds it to 2.6.18.
KERNEL_HAS_CLEAR_NLINK = $(shell \
	if grep "clear_nlink(struct inode \*inode)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_CLEAR_NLINK" ; \
	fi)
#
# KERNEL_HAS_CLEAR_NLINK Detection [END]

# KERNEL_HAS_WBC_NONRWRITEINDEXUPDATE Detection [START]
#
# Find out whether the kernel has a no_nrwrite_index_update field in struct
# writeback_control.
# Note: Existed in vanilla 2.6.28-2.6.34, but RedHat removes it from their 
# 2.6.32.
KERNEL_HAS_WBC_NONRWRITEINDEXUPDATE = $(shell \
	if grep "unsigned no_nrwrite_index_update" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/writeback.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_WRITEBACK_NONRWRITEINDEXUPDATE" ; \
	fi)
#
# KERNEL_HAS_WBC_NONRWRITEINDEXUPDATE Detection [END]


# KERNEL_HAS_TEST_LOCK_VOID_TWOARGS Detection [START]
#
# Find out whether kernel has posix_test_lock method with two args and void
# return value.
# Note: There were at least three different versions since 2.6.16 before this
# was introduced in vanilla 2.6.23
KERNEL_HAS_TEST_LOCK_VOID_TWOARGS = $(shell \
	if grep "void posix_test_lock(struct file \*, struct file_lock \*);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_TEST_LOCK_VOID_TWOARGS" ; \
	fi)
#
# KERNEL_HAS_TEST_LOCK_VOID_TWOARGS Detection [END]


# KERNEL_HAS_ATTR_OPEN Detection [START]
#
# Find out whether kernel defines ATTR_OPEN.
# Note: Was introduced in vanilla 2.6.24
KERNEL_HAS_ATTR_OPEN = $(shell \
	if grep "define ATTR_OPEN" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_ATTR_OPEN" ; \
	fi)
#
# KERNEL_HAS_ATTR_OPEN Detection [END]

# KERNEL_HAS_TAG_PAGES_FOR_WRITEBACK  Detection [START]
#
# Find out wheter the kernel has tag_pages_for_writeback(). 
# Introduced (in the header) in 2.6.37, in the C files already before 
KERNEL_HAS_TAG_PAGES_FOR_WRITEBACK = $(shell\
	if grep "tag_pages_for_writeback" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/writeback.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_TAG_PAGES_FOR_WRITEBACK" ; \
	fi)
#
# KERNEL_HAS_TAG_PAGES_FOR_WRITEBACK  Detection [END]

# KERNEL_HAS_WBC_TAGGED_WRITEPAGES  Detection [START]
#
# Find out wheter the kernel has wbc->tagged_writepages to prevent
# live deadlocks. Introduced in 3.1 and going to be backported
# to at least 3.0
KERNEL_HAS_WBC_TAGGED_WRITEPAGES = $(shell\
	if grep "tagged_writepages:1" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/writeback.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_WBC_TAGGED_WRITEPAGES" ; \
	fi)
#
# KERNEL_HAS_WBC_TAGGED_WRITEPAGES  Detection [END]

# KERNEL_HAS_IHOLD Detection [START]
#
# Find out whether the kernel has a ihold function.
# Note: Was added in vanilla 2.6.37, but RedHat adds it to their 2.6.32.
KERNEL_HAS_IHOLD = $(shell \
	if grep ' ihold(struct inode.*)' \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_IHOLD" ; \
	fi)
#
# KERNEL_HAS_IHOLD Detection [END]

# KERNEL_HAS_TASK_IO_ACCOUNTING Detection [START]
#
# Find out whether the kernel has task_io_account_read/write.
# Note: Was introduced in vanilla 2.6.20, but RedHat adds it to 2.6.18.
KERNEL_HAS_TASK_IO_ACCOUNTING = $(shell \
	if grep ' task_io_account_read(size_t .*)' \
		${KSRCDIR_PRUNED_HEAD}/include/linux/task_io_accounting_ops.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_TASK_IO_ACCOUNTING" ; \
	fi)
#
# KERNEL_HAS_TASK_IO_ACCOUNTING Detection [END]

# KERNEL_HAS_FSYNC_RANGE Detection [START]
#
# Find out whether the kernel has fsync start and end range arguments.
# Note: fsync start and end were added in vanilla 3.1, but SLES11SP3 adds it to 
#       its 3.0 kernel.
KERNEL_HAS_FSYNC_RANGE = $(shell \
	if grep "int (\*fsync) (struct file \*, loff_t, loff_t, int datasync);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_FSYNC_RANGE" ; \
	fi)
#
# KERNEL_HAS_FSYNC_RANGE Detection [END]

# KERNEL_HAS_INC_NLINK Detection [START]
#
# Find out whether the kernel has inc_nlink()
# Note: inc_nlink was added in vanilla 2.6.19, but RHEL5 also has it.
KERNEL_HAS_INC_NLINK = $(shell \
	if grep " inc_nlink(struct inode.*)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_INC_NLINK" ; \
	fi)
#
# KERNEL_HAS_INC_NLINK Detection [END]

# KERNEL_HAS_S_D_OP Detection [START]
#
# Find out whether the kernel has define struct dentry_operations *s_d_op
# in struct super_block. If it has it used that to check if the file system
# needs to revalidate dentries.
#
KERNEL_HAS_S_D_OP = $(shell \
	if grep "struct dentry_operations \*s_d_op;" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_S_D_OP" ; \
	fi)
# KERNEL_HAS_S_D_OP Detection [END]

# KERNEL_HAS_LOOKUP_CONTINUE Detection [START]
#
# Find out whether the kernel has define struct dentry_operations *s_d_op
# in struct super_block. If it has it used that to check if the file system
# needs to revalidate dentries.
#
KERNEL_HAS_LOOKUP_CONTINUE = $(shell \
	if grep "define LOOKUP_CONTINUE " \
		${KSRCDIR_PRUNED_HEAD}/include/linux/namei.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_LOOKUP_CONTINUE" ; \
	fi)
# KERNEL_HAS_LOOKUP_CONTINUE Detection [END]

# KERNEL_HAS_D_MATERIALISE_UNIQUE Detection [START]
#
# Find out whether the kernel has d_materialise_unique() to
# add dir dentries.
#
KERNEL_HAS_D_MATERIALISE_UNIQUE = $(shell \
	if grep "d_materialise_unique(struct dentry \*, struct inode \*)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/dcache.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_D_MATERIALISE_UNIQUE" ; \
	fi)
# KERNEL_HAS_D_MATERIALISE_UNIQUE Detection [END]

# KERNEL_HAS_PDE_DATA Detection [START]
#
# Find out whether the kernel has a PDE_DATA method.
#
# Note: This method was added in vanilla linux-3.10
#
KERNEL_HAS_PDE_DATA = $(shell \
	if grep "PDE_DATA(const struct inode \*)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/proc_fs.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_PDE_DATA" ; \
	fi)
# KERNEL_HAS_PDE_DATA Detection [END]

# KERNEL_HAS_CURRENT_NSPROXY Detection [START]
#
# Find out whether the kernel has current->nsproxy
#
# Note: This was added in vanilla linux-2.6.19
#
KERNEL_HAS_CURRENT_NSPROXY = $(shell \
	if grep "struct nsproxy \*nsproxy;" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/sched.h 1>/dev/null 2>&1 ; \
	then echo "-DKERNEL_HAS_CURRENT_NSPROXY" ; \
	fi)
# KERNEL_HAS_CURRENT_NSPROXY Detection [END]


# Combine results
KERNEL_FEATURE_DETECTION := $(KERNEL_HAS_IBLKSIZE) $(KERNEL_HAS_FLUSH_OWNER) \
	$(KERNEL_HAS_STATFS_DENTRY) $(KERNEL_HAS_DROP_NLINK) \
	$(KERNEL_HAS_CLEAR_NLINK) $(KERNEL_HAS_WBC_NONRWRITEINDEXUPDATE) \
	$(KERNEL_HAS_TEST_LOCK_VOID_TWOARGS) $(KERNEL_HAS_ATTR_OPEN) \
	$(KERNEL_HAS_TAG_PAGES_FOR_WRITEBACK) $(KERNEL_HAS_WBC_TAGGED_WRITEPAGES) \
	$(KERNEL_HAS_IHOLD) $(KERNEL_HAS_TASK_IO_ACCOUNTING) \
	$(KERNEL_HAS_FSYNC_RANGE) $(KERNEL_HAS_INC_NLINK) $(KERNEL_HAS_S_D_OP) \
	$(KERNEL_HAS_LOOKUP_CONTINUE) $(KERNEL_HAS_D_MATERIALISE_UNIQUE) \
	$(KERNEL_HAS_PDE_DATA) $(KERNEL_HAS_CURRENT_NSPROXY)



