#ifndef ACQUIREAPPENDLOCKMSG_H_
#define ACQUIREAPPENDLOCKMSG_H_

#include <common/net/message/NetMessage.h>

struct AcquireAppendLockMsg;
typedef struct AcquireAppendLockMsg AcquireAppendLockMsg;

static inline void AcquireAppendLockMsg_init(AcquireAppendLockMsg* this);
static inline void AcquireAppendLockMsg_initFromSession(AcquireAppendLockMsg* this,
   const char* sessionID, int fd);
static inline AcquireAppendLockMsg* AcquireAppendLockMsg_construct(void);
static inline AcquireAppendLockMsg* AcquireAppendLockMsg_constructFromSession(
   const char* sessionID, int fd);
static inline void AcquireAppendLockMsg_uninit(NetMessage* this);
static inline void AcquireAppendLockMsg_destruct(NetMessage* this);

// virtual functions
extern void AcquireAppendLockMsg_serializePayload(NetMessage* this, char* buf);
extern fhgfs_bool AcquireAppendLockMsg_deserializePayload(NetMessage* this,
   const char* buf, size_t bufLen);
extern unsigned AcquireAppendLockMsg_calcMessageLength(NetMessage* this);

// getters & setters
static inline const char* AcquireAppendLockMsg_getSessionID(AcquireAppendLockMsg* this);
static inline int AcquireAppendLockMsg_getFD(AcquireAppendLockMsg* this);


struct AcquireAppendLockMsg
{
   NetMessage netMessage;
   
   unsigned sessionIDLen;
   const char* sessionID;
   int fd;
};


void AcquireAppendLockMsg_init(AcquireAppendLockMsg* this)
{
   NetMessage_init( (NetMessage*)this, NETMSGTYPE_AcquireAppendLock);
   
   // assign virtual functions
   ( (NetMessage*)this)->uninit = AcquireAppendLockMsg_uninit;

   ( (NetMessage*)this)->serializePayload = AcquireAppendLockMsg_serializePayload;
   ( (NetMessage*)this)->deserializePayload = AcquireAppendLockMsg_deserializePayload;
   ( (NetMessage*)this)->calcMessageLength = AcquireAppendLockMsg_calcMessageLength;
}
   
/**
 * @param sessionID just a reference, so do not free it as long as you use this object!
 */
void AcquireAppendLockMsg_initFromSession(AcquireAppendLockMsg* this, const char* sessionID, int fd)
{
   AcquireAppendLockMsg_init(this);
   
   this->sessionID = sessionID;
   this->sessionIDLen = os_strlen(sessionID);
   
   this->fd = fd;
}

AcquireAppendLockMsg* AcquireAppendLockMsg_construct(void)
{
   struct AcquireAppendLockMsg* this = os_kmalloc(sizeof(struct AcquireAppendLockMsg) );
   
   AcquireAppendLockMsg_init(this);
   
   return this;
}

/**
 * @param sessionID just a reference, so do not free it as long as you use this object!
 */
AcquireAppendLockMsg* AcquireAppendLockMsg_constructFromSession(const char* sessionID, int fd)
{
   struct AcquireAppendLockMsg* this = os_kmalloc(sizeof(struct AcquireAppendLockMsg) );
   
   AcquireAppendLockMsg_initFromSession(this, sessionID, fd);
   
   return this;
}

void AcquireAppendLockMsg_uninit(NetMessage* this)
{
   NetMessage_uninit(this);
}

void AcquireAppendLockMsg_destruct(NetMessage* this)
{
   AcquireAppendLockMsg_uninit(this);
   
   os_kfree(this);
}

const char* AcquireAppendLockMsg_getSessionID(AcquireAppendLockMsg* this)
{
   return this->sessionID;
}

int AcquireAppendLockMsg_getFD(AcquireAppendLockMsg* this)
{
   return this->fd;
}



#endif /*ACQUIREAPPENDLOCKMSG_H_*/
