#ifndef RELEASEAPPENDLOCKMSG_H_
#define RELEASEAPPENDLOCKMSG_H_

#include <common/net/message/NetMessage.h>


struct ReleaseAppendLockMsg;
typedef struct ReleaseAppendLockMsg ReleaseAppendLockMsg;

static inline void ReleaseAppendLockMsg_init(ReleaseAppendLockMsg* this);
static inline void ReleaseAppendLockMsg_initFromSession(ReleaseAppendLockMsg* this,
   const char* sessionID, int fd);
static inline ReleaseAppendLockMsg* ReleaseAppendLockMsg_construct(void);
static inline ReleaseAppendLockMsg* ReleaseAppendLockMsg_constructFromSession(
   const char* sessionID, int fd);
static inline void ReleaseAppendLockMsg_uninit(NetMessage* this);
static inline void ReleaseAppendLockMsg_destruct(NetMessage* this);

// virtual functions
extern void ReleaseAppendLockMsg_serializePayload(NetMessage* this, char* buf);
extern fhgfs_bool ReleaseAppendLockMsg_deserializePayload(NetMessage* this, const char* buf, size_t bufLen);
extern unsigned ReleaseAppendLockMsg_calcMessageLength(NetMessage* this);

// getters & setters
static inline const char* ReleaseAppendLockMsg_getSessionID(ReleaseAppendLockMsg* this);
static inline int ReleaseAppendLockMsg_getFD(ReleaseAppendLockMsg* this);


struct ReleaseAppendLockMsg
{
   NetMessage netMessage;
   
   unsigned sessionIDLen;
   const char* sessionID;
   int fd;
};


void ReleaseAppendLockMsg_init(ReleaseAppendLockMsg* this)
{
   NetMessage_init( (NetMessage*)this, NETMSGTYPE_ReleaseAppendLock);
   
   // assign virtual functions
   ( (NetMessage*)this)->uninit = ReleaseAppendLockMsg_uninit;

   ( (NetMessage*)this)->serializePayload = ReleaseAppendLockMsg_serializePayload;
   ( (NetMessage*)this)->deserializePayload = ReleaseAppendLockMsg_deserializePayload;
   ( (NetMessage*)this)->calcMessageLength = ReleaseAppendLockMsg_calcMessageLength;
}
   
/**
 * @param sessionID just a reference, so do not free it as long as you use this object!
 */
void ReleaseAppendLockMsg_initFromSession(ReleaseAppendLockMsg* this, const char* sessionID, int fd)
{
   ReleaseAppendLockMsg_init(this);
   
   this->sessionID = sessionID;
   this->sessionIDLen = os_strlen(sessionID);
   
   this->fd = fd;
}

ReleaseAppendLockMsg* ReleaseAppendLockMsg_construct(void)
{
   struct ReleaseAppendLockMsg* this = os_kmalloc(sizeof(struct ReleaseAppendLockMsg) );
   
   ReleaseAppendLockMsg_init(this);
   
   return this;
}

/**
 * @param sessionID just a reference, so do not free it as long as you use this object!
 */
ReleaseAppendLockMsg* ReleaseAppendLockMsg_constructFromSession(const char* sessionID, int fd)
{
   struct ReleaseAppendLockMsg* this = os_kmalloc(sizeof(struct ReleaseAppendLockMsg) );
   
   ReleaseAppendLockMsg_initFromSession(this, sessionID, fd);
   
   return this;
}

void ReleaseAppendLockMsg_uninit(NetMessage* this)
{
   NetMessage_uninit( (NetMessage*)this);
}

void ReleaseAppendLockMsg_destruct(NetMessage* this)
{
   ReleaseAppendLockMsg_uninit( (NetMessage*)this);
   
   os_kfree(this);
}


const char* ReleaseAppendLockMsg_getSessionID(ReleaseAppendLockMsg* this)
{
   return this->sessionID;
}

int ReleaseAppendLockMsg_getFD(ReleaseAppendLockMsg* this)
{
   return this->fd;
}

#endif /*RELEASEAPPENDLOCKMSG_H_*/
