#ifndef STATRESPMSG_H_
#define STATRESPMSG_H_

#include <common/net/message/NetMessage.h>
#include <common/storage/StatData.h>


struct StatRespMsg;
typedef struct StatRespMsg StatRespMsg;

static inline void StatRespMsg_init(StatRespMsg* this);
static inline StatRespMsg* StatRespMsg_construct(void);
static inline void StatRespMsg_uninit(NetMessage* this);
static inline void StatRespMsg_destruct(NetMessage* this);

// virtual functions
extern fhgfs_bool StatRespMsg_deserializePayload(NetMessage* this, const char* buf, size_t bufLen);

// getters & setters
static inline int StatRespMsg_getResult(StatRespMsg* this);
static inline StatData* StatRespMsg_getStatData(StatRespMsg* this);


struct StatRespMsg
{
   NetMessage netMessage;
   
   int result;
   StatData statData;
};


void StatRespMsg_init(StatRespMsg* this)
{
   NetMessage_init( (NetMessage*)this, NETMSGTYPE_StatResp);
   
   // assign virtual functions
   ( (NetMessage*)this)->uninit = StatRespMsg_uninit;

   ( (NetMessage*)this)->serializePayload   = _NetMessage_serializeDummy;
   ( (NetMessage*)this)->deserializePayload = StatRespMsg_deserializePayload;
   ( (NetMessage*)this)->calcMessageLength  = _NetMessage_calcMessageLengthDummy;
}
   
StatRespMsg* StatRespMsg_construct(void)
{
   struct StatRespMsg* this = os_kmalloc(sizeof(*this) );
   
   StatRespMsg_init(this);
   
   return this;
}

void StatRespMsg_uninit(NetMessage* this)
{
   NetMessage_uninit( (NetMessage*)this);
}

void StatRespMsg_destruct(NetMessage* this)
{
   StatRespMsg_uninit( (NetMessage*)this);
   
   os_kfree(this);
}


int StatRespMsg_getResult(StatRespMsg* this)
{
   return this->result;
}


StatData* StatRespMsg_getStatData(StatRespMsg* this)
{
   return &this->statData;
}


#endif /*STATRESPMSG_H_*/
