#ifndef LISTDIRFROMOFFSETRESPMSG_H_
#define LISTDIRFROMOFFSETRESPMSG_H_

#include <common/net/message/NetMessage.h>
#include <common/Common.h>


struct ListDirFromOffsetRespMsg;
typedef struct ListDirFromOffsetRespMsg ListDirFromOffsetRespMsg;

static inline void ListDirFromOffsetRespMsg_init(ListDirFromOffsetRespMsg* this);
static inline void ListDirFromOffsetRespMsg_initFromResult(ListDirFromOffsetRespMsg* this,
   int result, StrCpyVec* names, IntCpyVec* entryTypes, int64_t newServerOffset);
static inline ListDirFromOffsetRespMsg* ListDirFromOffsetRespMsg_construct(void);
static inline ListDirFromOffsetRespMsg* ListDirFromOffsetRespMsg_constructFromResult(
   int result, StrCpyVec* names, IntCpyVec* entryTypes, int64_t newServerOffset);
static inline void ListDirFromOffsetRespMsg_uninit(NetMessage* this);
static inline void ListDirFromOffsetRespMsg_destruct(NetMessage* this);

// virtual functions
extern void ListDirFromOffsetRespMsg_serializePayload(NetMessage* this, char* buf);
extern fhgfs_bool ListDirFromOffsetRespMsg_deserializePayload(NetMessage* this, const char* buf,
   size_t bufLen);
extern unsigned ListDirFromOffsetRespMsg_calcMessageLength(NetMessage* this);

// inliners   
static inline void ListDirFromOffsetRespMsg_parseNames(ListDirFromOffsetRespMsg* this,
   StrCpyVec* outNames);
static inline void ListDirFromOffsetRespMsg_parseEntryTypes(ListDirFromOffsetRespMsg* this,
   IntCpyVec* outEntryTypes);

// getters & setters
static inline int ListDirFromOffsetRespMsg_getResult(ListDirFromOffsetRespMsg* this);
static inline uint64_t ListDirFromOffsetRespMsg_getNewServerOffset(ListDirFromOffsetRespMsg* this);


struct ListDirFromOffsetRespMsg
{
   NetMessage netMessage;
   
   int result;
   
   int64_t newServerOffset;

   // for serialization
   StrCpyVec* names; // not owned by this object!
   IntCpyVec* entryTypes; // not owned by this object! (optional, only filled if requested)
   
   // for deserialization
   unsigned namesElemNum;
   const char* namesListStart;
   unsigned namesBufLen;
   unsigned entryTypesElemNum;
   const char* entryTypesListStart;
   unsigned entryTypesBufLen;
};


void ListDirFromOffsetRespMsg_init(ListDirFromOffsetRespMsg* this)
{
   NetMessage_init( (NetMessage*)this, NETMSGTYPE_ListDirFromOffsetResp);
   
   // assign virtual functions
   ( (NetMessage*)this)->uninit = ListDirFromOffsetRespMsg_uninit;

   ( (NetMessage*)this)->serializePayload = ListDirFromOffsetRespMsg_serializePayload;
   ( (NetMessage*)this)->deserializePayload = ListDirFromOffsetRespMsg_deserializePayload;
   ( (NetMessage*)this)->calcMessageLength = ListDirFromOffsetRespMsg_calcMessageLength;
}
   
/**
 * @param names just a reference, so do not free it as long as you use this object!
 */
void ListDirFromOffsetRespMsg_initFromResult(ListDirFromOffsetRespMsg* this, int result,
   StrCpyVec* names, IntCpyVec* entryTypes, int64_t newServerOffset)
{
   ListDirFromOffsetRespMsg_init(this);
   
   this->result = result;
   this->names = names;
   this->entryTypes = entryTypes;
   this->newServerOffset = newServerOffset;
}

ListDirFromOffsetRespMsg* ListDirFromOffsetRespMsg_construct(void)
{
   struct ListDirFromOffsetRespMsg* this = os_kmalloc(sizeof(*this) );
   
   ListDirFromOffsetRespMsg_init(this);
   
   return this;
}

/**
 * @param names just a reference, so do not free it as long as you use this object!
 */
ListDirFromOffsetRespMsg* ListDirFromOffsetRespMsg_constructFromResult(int result,
   StrCpyVec* names, IntCpyVec* entryTypes, int64_t newServerOffset)
{
   struct ListDirFromOffsetRespMsg* this = os_kmalloc(sizeof(*this) );
   
   ListDirFromOffsetRespMsg_initFromResult(this, result, names, entryTypes, newServerOffset);
   
   return this;
}

void ListDirFromOffsetRespMsg_uninit(NetMessage* this)
{
   NetMessage_uninit( (NetMessage*)this);
}

void ListDirFromOffsetRespMsg_destruct(NetMessage* this)
{
   ListDirFromOffsetRespMsg_uninit( (NetMessage*)this);
   
   os_kfree(this);
}


void ListDirFromOffsetRespMsg_parseNames(ListDirFromOffsetRespMsg* this, StrCpyVec* outNames)
{
   Serialization_deserializeStrCpyVec(
      this->namesBufLen, this->namesElemNum, this->namesListStart, outNames);
}

void ListDirFromOffsetRespMsg_parseEntryTypes(ListDirFromOffsetRespMsg* this,
   IntCpyVec* outEntryTypes)
{
   Serialization_deserializeIntCpyVec(
      this->entryTypesBufLen, this->entryTypesElemNum, this->entryTypesListStart, outEntryTypes);
}

int ListDirFromOffsetRespMsg_getResult(ListDirFromOffsetRespMsg* this)
{
   return this->result;
}

uint64_t ListDirFromOffsetRespMsg_getNewServerOffset(ListDirFromOffsetRespMsg* this)
{
   return this->newServerOffset;
}


#endif /*LISTDIRFROMOFFSETRESPMSG_H_*/
