#ifndef OPEN_STANDARDSOCKET_H_
#define OPEN_STANDARDSOCKET_H_

#include <common/external/sdp_inet.h>
#include <common/toolkit/StringTk.h>
#include <common/toolkit/Time.h>
#include <common/Common.h>
#include <common/net/sock/PooledSocket.h>


struct StandardSocket;
typedef struct StandardSocket StandardSocket;


extern void StandardSocket_init(StandardSocket* this, int domain, int type, int protocol);
extern StandardSocket* StandardSocket_construct(int domain, int type, int protocol);
extern StandardSocket* StandardSocket_constructUDP(void);
extern StandardSocket* StandardSocket_constructTCP(void);
extern StandardSocket* StandardSocket_constructSDP(void);
extern void _StandardSocket_uninit(Socket* this);
extern void _StandardSocket_destruct(Socket* this);

extern fhgfs_bool StandardSocket_createSocketPair(int domain, int type, int protocol,
   StandardSocket** outEndpointA, StandardSocket** outEndpointB);

extern fhgfs_bool StandardSocket_setSoKeepAlive(StandardSocket* this, fhgfs_bool enable);
extern fhgfs_bool StandardSocket_setSoBroadcast(StandardSocket* this, fhgfs_bool enable);
extern fhgfs_bool StandardSocket_setSoReuseAddr(StandardSocket* this, fhgfs_bool enable);
extern fhgfs_bool StandardSocket_getSoRcvBuf(StandardSocket* this, int* outSize);
extern fhgfs_bool StandardSocket_setSoRcvBuf(StandardSocket* this, int size);
extern fhgfs_bool StandardSocket_setTcpNoDelay(StandardSocket* this, fhgfs_bool enable);

//extern fhgfs_bool StandardSocket_connectByName(StandardSocket* this, const char* hostname,
//   unsigned short port, ExternalHelperd* helperd);
extern fhgfs_bool _StandardSocket_connectByIP(Socket* this, fhgfs_in_addr* ipaddress,
   unsigned short port);
extern fhgfs_bool _StandardSocket_bindToAddr(Socket* this, fhgfs_in_addr* ipaddress,
   unsigned short port);
extern fhgfs_bool _StandardSocket_listen(Socket* this);
extern fhgfs_bool _StandardSocket_shutdown(Socket* this);
extern fhgfs_bool _StandardSocket_shutdownAndRecvDisconnect(Socket* this, int timeoutMS);

extern ssize_t _StandardSocket_recv(Socket* this, void *buf, size_t len, int flags);
extern ssize_t _StandardSocket_recvT(Socket* this, void *buf, size_t len, int flags,
   int timeoutMS);

extern ssize_t _StandardSocket_send(Socket* this, const void *buf, size_t len, int flags);
extern ssize_t _StandardSocket_sendto(Socket* this, const void *buf, size_t len, int flags,
   fhgfs_sockaddr_in *to);

extern ssize_t StandardSocket_sendtoIP(StandardSocket* this, const void *buf, size_t len, int flags,
   fhgfs_in_addr ipAddr, unsigned short port);

extern ssize_t StandardSocket_recvfrom(StandardSocket* this, void *buf, size_t len,
   int flags, fhgfs_sockaddr_in *from);
extern ssize_t StandardSocket_recvfromT(StandardSocket* this, void *buf, size_t len,
   int flags, fhgfs_sockaddr_in *from, int timeoutMS);

extern ssize_t StandardSocket_broadcast(StandardSocket* this, void *buf, size_t len, int flags,
   fhgfs_in_addr* broadcastIP, unsigned short port);


extern fhgfs_bool _StandardSocket_initSock(StandardSocket* this, int domain, int type,
   int protocol);
extern void __StandardSocket_setAllocMode(StandardSocket* this, gfp_t flags);
extern int _StandardSocket_setsockopt(StandardSocket* this, int level, int optname, char* optval,
   int optlen);
extern int _StandardSocket_getsockopt(StandardSocket* this, int level, int optname,
   char *optval, int *optlen);

// getters & setters
static inline unsigned short StandardSocket_getSockDomain(StandardSocket* this);
static inline struct socket* StandardSocket_getRawSock(StandardSocket* this);

// inliners
static inline fhgfs_bool StandardSocket_getSockValid(StandardSocket* this);
static inline char* StandardSocket_getPeername(StandardSocket* this);
static inline unsigned short StandardSocket_getSockDomain(StandardSocket* this);
static inline struct socket* StandardSocket_getRawSock(StandardSocket* this);


struct StandardSocket
{
   PooledSocket pooledSocket;

   struct socket* sock;
   unsigned short sockDomain;
};


fhgfs_bool StandardSocket_getSockValid(StandardSocket* this)
{
   Socket* thisBase = (Socket*)this;

   return thisBase->sockValid;
}

char* StandardSocket_getPeername(StandardSocket* this)
{
   Socket* thisBase = (Socket*)this;

   return thisBase->peername;
}

unsigned short StandardSocket_getSockDomain(StandardSocket* this)
{
   return this->sockDomain;
}

struct socket* StandardSocket_getRawSock(StandardSocket* this)
{
   return this->sock;
}


#endif /*OPEN_STANDARDSOCKET_H_*/
