/*
 * Information provided by stat()
 */

#include <common/toolkit/Serialization.h>
#include "StatData.h"

fhgfs_bool StatData_deserialize(const char* buf, size_t bufLen, StatData* outThis, unsigned* outLen)
{

   /* TODO: All those integers could be deserialized using a type conversion
    * (my_struct) (buf), which would simplify the code and also be faster.
    * The data just could not be used between architectures, or we would need to introduce
    * endian conversions */

   unsigned bufPos = 0;

   { // creationTime
      unsigned createTimeFieldLen;
      if(!Serialization_deserializeInt64(&buf[bufPos], bufLen-bufPos,
         &outThis->creationTimeSecs, &createTimeFieldLen) )
         return fhgfs_false;

      bufPos += createTimeFieldLen;
   }

   { // aTime
      unsigned aTimeFieldLen;
      if(!Serialization_deserializeInt64(&buf[bufPos], bufLen-bufPos,
         &outThis->settableFileAttribs.lastAccessTimeSecs, &aTimeFieldLen) )
         return fhgfs_false;

      bufPos += aTimeFieldLen;
   }

   { // mtime
      unsigned mTimeFieldLen;
      if(!Serialization_deserializeInt64(&buf[bufPos], bufLen-bufPos,
         &outThis->settableFileAttribs.modificationTimeSecs, &mTimeFieldLen) )
         return fhgfs_false;

      bufPos += mTimeFieldLen;
   }

   { // ctime
      unsigned cTimeFieldLen;
      if(!Serialization_deserializeInt64(&buf[bufPos], bufLen-bufPos,
         &outThis->attribChangeTimeSecs, &cTimeFieldLen) )
         return fhgfs_false;

      bufPos += cTimeFieldLen;
   }

   { // fileSize
      unsigned sizeFieldLen;
      if(!Serialization_deserializeInt64(&buf[bufPos], bufLen-bufPos,
         &outThis->fileSize, &sizeFieldLen) )
         return fhgfs_false;

      bufPos += sizeFieldLen;
   }

   { // nlink
      unsigned nlinkFieldLen;

      if(!Serialization_deserializeUInt(&buf[bufPos], bufLen-bufPos,
         &outThis->nlink, &nlinkFieldLen) )
         return fhgfs_false;

      bufPos += nlinkFieldLen;
   }

   { // contentsVersion
      unsigned contentsFieldLen;

      if(!Serialization_deserializeUInt(&buf[bufPos], bufLen-bufPos,
         &outThis->contentsVersion, &contentsFieldLen) )
         return fhgfs_false;

      bufPos += contentsFieldLen;
   }

   { // uid
      unsigned uidFieldLen;
      if(!Serialization_deserializeUInt(&buf[bufPos], bufLen-bufPos,
         &outThis->settableFileAttribs.userID, &uidFieldLen) )
         return fhgfs_false;

      bufPos += uidFieldLen;
   }

   { // gid
      unsigned gidFieldLen;
      if(!Serialization_deserializeUInt(&buf[bufPos], bufLen-bufPos,
         &outThis->settableFileAttribs.groupID, &gidFieldLen) )
         return fhgfs_false;

      bufPos += gidFieldLen;
   }

   { // mode
      unsigned modeFieldLen;
      if(!Serialization_deserializeInt(&buf[bufPos], bufLen-bufPos,
         &outThis->settableFileAttribs.mode, &modeFieldLen) )
         return fhgfs_false;

      bufPos += modeFieldLen;
   }

   *outLen = bufPos;

   return fhgfs_true;
}



