/*
 * Information provided by stat()
 */

#ifndef STATDATA_H_
#define STATDATA_H_

#include <common/storage/StorageDefinitions.h>
#include <common/FhgfsTypes.h>

struct StatData;
typedef struct StatData StatData;

extern fhgfs_bool StatData_deserialize(const char* buf, size_t bufLen,
   StatData* outThis, unsigned* outLen);

static inline void StatData_getOsStat(StatData* this, fhgfs_stat* outOsStat);

struct StatData
{

      int64_t fileSize;
      int64_t creationTimeSecs;     // real creation time
      int64_t attribChangeTimeSecs; // this corresponds to unix ctime
      unsigned nlink;

      SettableFileAttribs settableFileAttribs;

      unsigned contentsVersion; // inc'ed when file contents are modified (for cache invalidation)

};

void StatData_getOsStat(StatData* this, fhgfs_stat* outOsStat)
{
   outOsStat->mode  = this->settableFileAttribs.mode;
   outOsStat->size  = this->fileSize;
   outOsStat->uid   = this->settableFileAttribs.userID;
   outOsStat->gid   = this->settableFileAttribs.groupID;
   outOsStat->nlink = this->nlink;

   outOsStat->atime.tv_sec   = this->settableFileAttribs.lastAccessTimeSecs;
   outOsStat->atime.tv_nsec  = 0;

   outOsStat->mtime.tv_sec   = this->settableFileAttribs.modificationTimeSecs;
   outOsStat->mtime.tv_nsec  = 0;

   outOsStat->ctime.tv_sec   = this->attribChangeTimeSecs;
   outOsStat->ctime.tv_nsec  = 0;
}

#endif /* STATDATA_H_ */
