#ifndef METADATATK_H_
#define METADATATK_H_

#include <app/App.h>
#include <common/Common.h>
#include <common/nodes/Node.h>
#include <common/storage/Path.h>
#include <common/storage/StorageErrors.h>
#include <common/storage/EntryInfo.h>
#include <common/storage/StorageErrors.h>
#include <common/storage/StorageDefinitions.h>
#include <common/toolkit/list/UInt16List.h>
#include <common/toolkit/MetadataTk.h>

#include <linux/fs.h>

#define METADATATK_OWNERSEARCH_MAX_STEPS    128 /* to avoid infinite searching */


struct CreateInfo;
typedef struct CreateInfo CreateInfo;

struct LookupCreateStatInfoIn;
typedef struct LookupCreateStatInfoIn LookupCreateStatInfoIn;
struct LookupCreateStatInfoOut;
typedef struct LookupCreateStatInfoOut LookupCreateStatInfoOut;


extern FhgfsOpsErr MetadataTk_validateAndStatSubentry(App* app, const EntryInfo* parentEntryInfo,
   const char* entryName, const EntryInfo* entryInfo, FhgfsOpsErr* outStatRes,
   fhgfs_stat* outFhgfsStat);
extern FhgfsOpsErr MetadataTk_lookupAndStatSubentry(App* app, const EntryInfo* parentEntryInfo,
   const char* entryName, EntryInfo* outEntryInfo, FhgfsOpsErr* outStatRes,
   fhgfs_stat* outFhgfsStat);
extern FhgfsOpsErr MetadataTk_lookupCreateStatFile(App* app, const LookupCreateStatInfoIn* inInfo,
   LookupCreateStatInfoOut* outInfo);

extern void MetadataTk_getRootEntryInfoCopy(App* app, EntryInfo* outEntryInfo);

// inliners

void CreateInfo_init(App* app, struct inode* parentDirInode, const char* entryName,
   int mode, struct CreateInfo* outCreateInfo);
static inline void LookupCreateStatInfoIn_init(LookupCreateStatInfoIn* this,
   const EntryInfo* parentEntryInfo, const CreateInfo* createInfo, fhgfs_bool isExclusiveCreate);
static inline void LookupCreateStatInfoOut_prepare(LookupCreateStatInfoOut* this,
   EntryInfo* outEntryInfo, fhgfs_stat* outFhgfsStat);



/**
 * File/Dir create information. Should be initialized using CreateInfo_init().
 */
struct CreateInfo
{
      const char* entryName; // file name
      unsigned userID;
      unsigned groupID;
      int mode;
      UInt16List* preferredMetaTargets;
      UInt16List* preferredStorageTargets;
};

/**
 * In-Args for _lookupCreateStat operations. Init only with LookupCreateStatInfoIn_init().
 */
struct LookupCreateStatInfoIn
{
   const EntryInfo* parentEntryInfo;
   const CreateInfo* createInfo;
   fhgfs_bool isExclusiveCreate; // fhgfs_true iff O_EXCL is set
};

/**
 * Out-Args for _lookupCreateStat operations. Init only with LookupCreateStatInfoOut_prepare().
 *
 * Note: This needs preparation by the same caller that does init of LookupCreateStatInfoIn
 * (to assign some out-pointers before the actual method is called).
 */
struct LookupCreateStatInfoOut
{
   EntryInfo* entryInfo;
   fhgfs_stat* fhgfsStat;
   FhgfsOpsErr statRes;
};


static inline void LookupCreateStatInfoIn_init(LookupCreateStatInfoIn* this,
   const EntryInfo* parentEntryInfo, const CreateInfo* createInfo, fhgfs_bool isExclusiveCreate)
{
   this->parentEntryInfo = parentEntryInfo;
   this->createInfo = createInfo;
   this->isExclusiveCreate = isExclusiveCreate;
}


static inline void LookupCreateStatInfoOut_prepare(LookupCreateStatInfoOut* this,
   EntryInfo* outEntryInfo, fhgfs_stat* outFhgfsStat)
{
   // note: we only assign the pointers to the out-structs here

   this->entryInfo = outEntryInfo;
   this->fhgfsStat = outFhgfsStat;
}



#endif /*METADATATK_H_*/
