#include <common/toolkit/StringTk.h>
#include <common/Common.h>


void StringTk_explode(const char* s, char delimiter, StrCpyList* outList)
{
   ssize_t sLen = strlen(s);
   ssize_t lastPos = (strlen(s) && (s[0] == delimiter) ) ? 0 : -1;
   ssize_t currentPos;
   char* findRes;
   ssize_t newElemLen;

   // note: starts at pos 1 because an occurence at pos 0 would lead to an empty string,
   // which would be ignored anyways
   // note: i think the initialization of currentPos here is useless
   for(currentPos = 1; ; )
   {
      findRes = strchr(&s[lastPos+1], delimiter);

      if(!findRes)
      {
         ssize_t restLen = sLen - (lastPos+1);

         if(restLen)
         { // add rest
            char* newElem = StringTk_subStr(&s[lastPos+1], restLen);

            StrCpyList_append(outList, newElem);
            kfree(newElem);
         }

         return;
      }

      currentPos = findRes - s;

      // add substring to outList (leave the delimiter positions out)
      newElemLen = currentPos-lastPos-1; // -1=last delimiter
      if(newElemLen)
      {
         char* newElem = StringTk_subStr(&s[lastPos+1], newElemLen);

         StrCpyList_append(outList, newElem);
         kfree(newElem);
      }

      lastPos = currentPos;
   }
}

char* StringTk_strncpyTerminated(char* dest, const char* src, size_t count)
{
   // Note: The problem with strncpy() is that dest is not guaranteed to be zero-terminated.
   // strlcpy() does guarantee that.

   strlcpy(dest, src, count);

   return dest;
}

int StringTk_strToInt(const char* s)
{
   return (int)simple_strtol(s, NULL, 10);
}

unsigned StringTk_strToUInt(const char* s)
{
   return (unsigned)simple_strtoul(s, NULL, 10);
}

int64_t StringTk_strToInt64(const char* s)
{
   // note: other than the unsigned version, simple_strtoll is not exported in some kernel versions
   // and hence we use an emulation here

   if(*s == '-')
      return -StringTk_strToUInt64(&s[1] );

   return StringTk_strToUInt64(s);
}

uint64_t StringTk_strToUInt64(const char* s)
{
   return simple_strtoull(s, NULL, 10);
}

fhgfs_bool StringTk_strToBool(const char* s)
{
   if( !StringTk_hasLength(s) ||
       !strcmp(s, "1") ||
       !strnicmp(s, "y", 2) ||
       !strnicmp(s, "on", 3) ||
       !strnicmp(s, "yes", 4) ||
       !strnicmp(s, "true", 5) )
      return fhgfs_true;

   return fhgfs_false;
}

char* StringTk_strDup(const char* s)
{
   return kstrdup(s, GFP_NOFS);
}

/**
 * @return string is kmalloc'ed and has to be kfree'd by the caller
 */
char* StringTk_subStr(const char* start, unsigned numChars)
{
   char* subStr = (char*)os_kmalloc(numChars+1);

   if(numChars)
      memcpy(subStr, start, numChars);

   subStr[numChars] = 0;

   return subStr;
}


/**
 * @return string is kmalloc'ed and has to be kfree'd by the caller
 */
char* StringTk_trimCopy(const char* s)
{
   int lastRight;
   int firstLeft;
   int sLen;

   sLen = strlen(s);

   firstLeft = 0;
   while( (firstLeft < sLen) &&
      (s[firstLeft]==' ' || s[firstLeft]=='\n' || s[firstLeft]=='\r' || s[firstLeft]=='\t') )
      firstLeft++;

   if(firstLeft == sLen)
      return StringTk_strDup(""); // the string is empty or contains only space chars

   // the string contains at least one non-space char

   lastRight = sLen - 1;
   while(s[lastRight]==' ' || s[lastRight]=='\n' || s[lastRight]=='\r' || s[lastRight]=='\t')
      lastRight--;

   return StringTk_subStr(&s[firstLeft], lastRight - firstLeft + 1);
}

/**
 * Note: Provided, because old kernels don't provide kasprintf().
 *
 * @return will be kalloced and needs to be kfree'd by the caller
 */
char* StringTk_kasprintf(const char *fmt, ...)
{
   char* printStr;
   int printLen;
   char tmpChar; // (old kernels don't seem to be able to handle vsnprintf(NULL, 0, ...) )
   va_list ap;
   va_list apCopy; // kasprintf uses this

   va_start(ap, fmt);

   va_copy(apCopy, ap);

   printLen = vsnprintf(&tmpChar, 1, fmt, apCopy);

   va_end(apCopy);

   printStr = os_kmalloc(printLen + 1);
   if(likely(printStr) )
   {
      vsnprintf(printStr, printLen + 1, fmt, ap);
   }

   va_end(ap);

   return printStr;
}

char* StringTk_intToStr(int a)
{
   char aStr[24];
   sprintf(aStr, "%d", a);

   return StringTk_strDup(aStr);
}

char* StringTk_uintToStr(unsigned a)
{
   char aStr[24];
   sprintf(aStr, "%u", a);

   return StringTk_strDup(aStr);
}

char* StringTk_uintToHexStr(unsigned a)
{
   char aStr[24];
   sprintf(aStr, "%X", a);

   return StringTk_strDup(aStr);
}

char* StringTk_int64ToStr(int64_t a)
{
   char aStr[24];
   sprintf(aStr, "%lld", (long long)a);

   return StringTk_strDup(aStr);
}

char* StringTk_uint64ToHexStr(uint64_t a)
{
   char aStr[24];
   sprintf(aStr, "%llX", (unsigned long long)a);

   return StringTk_strDup(aStr);
}

char* StringTk_doubleToStr(double a)
{
   char aStr[32];
   sprintf(aStr, "%0.3f", a);

   return StringTk_strDup(aStr);
}

