#ifndef OPEN_TIME_H_
#define OPEN_TIME_H_

#include <common/Common.h>
#include <opentk/common/toolkit/OpenTk_Time.h>

/**
 * This is just a wrapper to provide more friendly names for the fhgfs_Time_xyz()-methods.
 * The basis for this is a monotonic clock.
 */

static inline void Time_setZero(Time* this);
static inline void Time_init(Time* this);
static inline void Time_initFromOther(Time* this, Time* other);
static inline Time* Time_construct(void);
static inline void Time_uninit(Time* this);
static inline void Time_destruct(Time* this);

static inline void Time_setToNow(Time* this);

static inline void Time_setFromOther(Time* this, Time* other);
static inline int Time_elapsedSinceMS(Time* this, Time* earlierT);
static inline int Time_elapsedSinceMicro(Time* this, Time* earlierT);
static inline int Time_elapsedMS(Time* this);

// operators
static inline fhgfs_bool Time_equals(Time* this, Time* other);
static inline fhgfs_bool Time_notequals(Time* this, Time* other);

void Time_setZero(Time* this)
{
   this->now.tv_sec = 0;
   this->now.tv_nsec = 0;
}

void Time_init(Time* this)
{
   fhgfs_Time_init(this);
}

void Time_initFromOther(Time* this, Time* other)
{
   fhgfs_Time_initFromOther(this, other);
}


Time* Time_construct(void)
{
   return fhgfs_Time_construct();
}


void Time_uninit(Time* this)
{
   fhgfs_Time_uninit(this);
}


void Time_destruct(Time* this)
{
   fhgfs_Time_destruct(this);
}



void Time_setToNow(Time* this)
{
   fhgfs_Time_setToNow(this);
}



void Time_setFromOther(Time* this, Time* other)
{
   fhgfs_Time_setFromOther(this, other);
}


int Time_elapsedSinceMS(Time* this, Time* earlierT)
{
   return fhgfs_Time_elapsedSinceMS(this, earlierT);
}


int Time_elapsedSinceMicro(Time* this, Time* earlierT)
{
   return fhgfs_Time_elapsedSinceMicro(this, earlierT);
}


int Time_elapsedMS(Time* this)
{
   return fhgfs_Time_elapsedMS(this);
}



// operators
fhgfs_bool Time_equals(Time* this, Time* other)
{
   return fhgfs_Time_equals(this, other);
}


fhgfs_bool Time_notequals(Time* this, Time* other)
{
   return fhgfs_Time_notequals(this, other);
}


#endif /* OPEN_TIME_H_ */
