#ifndef HEARTBEATMANAGER_H_
#define HEARTBEATMANAGER_H_

#include <app/config/Config.h>
#include <common/net/message/NetMessage.h>
#include <common/nodes/TargetMapper.h>
#include <nodes/NodeStoreEx.h>
#include <toolkit/ExternalHelperd.h>
#include <app/log/Logger.h>
#include <common/threading/Thread.h>
#include <common/threading/Mutex.h>
#include <common/threading/Condition.h>
#include "DatagramListener.h"


struct HeartbeatManager;
typedef struct HeartbeatManager HeartbeatManager;


extern void HeartbeatManager_init(HeartbeatManager* this, App* app, DatagramListener* dgramLis);
extern HeartbeatManager* HeartbeatManager_construct(App* app, DatagramListener* dgramLis);
extern void HeartbeatManager_uninit(HeartbeatManager* this);
extern void HeartbeatManager_destruct(HeartbeatManager* this);

extern fhgfs_bool HeartbeatManager_waitForMgmtInit(HeartbeatManager* this, int timeoutMS);


extern void __HeartbeatManager_run(Thread* this);
extern void __HeartbeatManager_requestLoop(HeartbeatManager* this);

extern void __HeartbeatManager_mgmtInit(HeartbeatManager* this);
extern fhgfs_bool __HeartbeatManager_waitForMgmtHeartbeat(HeartbeatManager* this);
extern void __HeartbeatManager_downloadAndSyncNodes(HeartbeatManager* this);
extern void __HeartbeatManager_printSyncResults(HeartbeatManager* this, NodeType nodeType,
   UInt16List* addedNodes, UInt16List* removedNodes);
extern void __HeartbeatManager_downloadAndSyncTargetMappings(HeartbeatManager* this);
extern fhgfs_bool __HeartbeatManager_registerNode(HeartbeatManager* this);
extern void __HeartbeatManager_reregisterNode(HeartbeatManager* this);
extern fhgfs_bool __HeartbeatManager_unregisterNode(HeartbeatManager* this);
extern void __HeartbeatManager_signalMgmtInitDone(HeartbeatManager* this);


// getters & setters
static inline fhgfs_bool HeartbeatManager_getValid(HeartbeatManager* this);


struct HeartbeatManager
{
   Thread thread;

   fhgfs_bool componentValid;

   App* app;
   Config* cfg;
   DatagramListener* dgramLis;
   ExternalHelperd* helperd;

   NodeStoreEx* mgmtNodes;
   NodeStoreEx* metaNodes;
   NodeStoreEx* storageNodes;
   
   fhgfs_bool nodeRegistered; // true if the mgmt host ack'ed our heartbeat
   
   fhgfs_bool mgmtInitDone;
   Mutex* mgmtInitDoneMutex;
   Condition* mgmtInitDoneCond; // signaled when init is done (doesn't mean it was successful)
};



fhgfs_bool HeartbeatManager_getValid(HeartbeatManager* this)
{
   return this->componentValid;
}


#endif /*HEARTBEATMANAGER_H_*/
