#ifndef FHGFSOPSEXPORT_H_
#define FHGFSOPSEXPORT_H_

#include <common/Common.h> // (placed up here for LINUX_VERSION_CODE definition)


/**
 * NFS export is probably not worth the backport efforts for kernels before 2.6.29
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)


#include <linux/exportfs.h>


struct FhgfsNfsFileHandle;
typedef struct FhgfsNfsFileHandle FhgfsNfsFileHandle;

extern const struct export_operations fhgfs_export_ops;


#if LINUX_VERSION_CODE < KERNEL_VERSION(3,5,0)
int FhgfsOpsExport_encodeNfsFileHandle(struct dentry* dentry, __u32* file_handle_buf, int* max_len,
   int connectable);
#else
int FhgfsOpsExport_encodeNfsFileHandle(struct inode* inode, __u32* file_handle_buf, int* max_len,
   struct inode* parent_inode);
#endif
struct dentry* FhgfsOpsExport_nfsFileHandleToDentry(struct super_block *sb, struct fid *fid,
   int fh_len, int fileid_type);


struct dentry* __FhgfsOpsExport_lookupDentryFromNfsHandle(struct super_block *sb,
   FhgfsNfsFileHandle* fhgfsNfsHandle);

fhgfs_bool __FhgfsOpsExport_parseEntryIDForNfsHandle(const char* entryID, uint32_t* outCounter,
   uint32_t* outTimestamp, uint16_t* outNodeID);
fhgfs_bool __FhgfsOpsExport_entryIDFromNfsHandle(uint32_t counter, uint32_t timestamp,
   uint16_t nodeID, char** outEntryID);



/**
 * The structure of the handle that will be encoded by _encodeFileHandle().
 *
 * EntryID string format (except for "root"): <u32hex_counter>-<u32hex_timestamp>-<u16hex_nodeID>
 */
struct FhgfsNfsFileHandle
{
   // note: fields not in natural order for better alignment and packed size

   uint32_t parentEntryIDCounter;    // from EntryInfo::parentEntryID
   uint32_t parentEntryIDTimestamp;  // from EntryInfo::parentEntryID
   uint32_t entryIDCounter;          // from EntryInfo::entryID
   uint32_t entryIDTimestamp;        // from EntryInfo::entryID
   uint16_t parentEntryIDNodeID;     // from EntryInfo::parentEntryID
   uint16_t entryIDNodeID;           // from EntryInfo::entryID
   uint16_t ownerNodeID;             // from EntryInfo::ownerNodeID
   char     entryType;               // from EntryInfo::entryType

} __attribute__((packed));


#endif // LINUX_VERSION_CODE

#endif /* FHGFSOPSEXPORT_H_ */
