#ifndef FHGFSOPSINODE_H_
#define FHGFSOPSINODE_H_

#include <app/App.h>
#include <common/threading/Mutex.h>
#include <common/toolkit/Time.h>
#include <net/filesystem/FhgfsOpsRemoting.h>
#include <filesystem/FsDirInfo.h>
#include <filesystem/FsFileInfo.h>
#include <toolkit/NoAllocBufferStore.h>
#include <common/Common.h>
#include "FhgfsInode.h"
#include "FhgfsOps_versions.h"

#include <linux/fs.h>
#include <linux/vfs.h>


// forward declaration
struct App;

struct FhgfsInodeComparisonInfo;
typedef struct FhgfsInodeComparisonInfo FhgfsInodeComparisonInfo;


#if LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
   extern struct dentry* FhgfsOps_lookupIntent(struct inode* parentDir, struct dentry* dentry,
      struct nameidata* nameidata);
#else
   extern struct dentry* FhgfsOps_lookupIntent(struct inode* parentDir, struct dentry* dentry,
      unsigned flags);
#endif // LINUX_VERSION_CODE

extern int FhgfsOps_getattr(struct vfsmount* mnt, struct dentry* dentry, struct kstat* kstat);
extern int FhgfsOps_setattr(struct dentry* dentry, struct iattr* iattr);

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0)
   extern int FhgfsOps_mkdir(struct inode* dir, struct dentry* dentry, int mode);
   extern int FhgfsOps_mknod(struct inode* dir, struct dentry* dentry, int mode, dev_t dev);
#else
   extern int FhgfsOps_mkdir(struct inode* dir, struct dentry* dentry, umode_t mode);
   extern int FhgfsOps_mknod(struct inode* dir, struct dentry* dentry, umode_t mode, dev_t dev);
#endif // LINUX_VERSION_CODE

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,3,0)
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, int mode,
      struct nameidata* nameidata);
#elif LINUX_VERSION_CODE < KERNEL_VERSION(3,6,0)
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, umode_t mode,
      struct nameidata* nameidata);
#else
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, umode_t mode,
      bool isExclusiveCreate);
#endif // LINUX_VERSION_CODE


extern int FhgfsOps_rmdir(struct inode* dir, struct dentry* dentry);
extern int FhgfsOps_unlink(struct inode* dir, struct dentry* dentry);

extern int FhgfsOps_symlink(struct inode* dir, struct dentry* dentry, const char* to);

extern int FhgfsOps_link(struct dentry* dentryFrom, struct inode* inode, struct dentry* dentryTo);
extern int FhgfsOps_hardlinkAsSymlink(struct dentry* oldDentry, struct inode* dir,
   struct dentry* newDentry);


extern void* FhgfsOps_follow_link(struct dentry* dentry, struct nameidata* nd);
extern void FhgfsOps_put_link(struct dentry *dentry, struct nameidata* nd, void* p);

extern int FhgfsOps_rename(struct inode* inodeDirFrom, struct dentry* dentryFrom,
   struct inode* inodeDirTo, struct dentry* dentryTo);

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0)
extern void FhgfsOps_truncate(struct inode* inode);
#endif // LINUX_VERSION_CODE

extern int FhgfsOps_vmtruncate(struct inode* inode, loff_t offset);


extern fhgfs_bool FhgfsOps_initInodeCache(void);
extern void FhgfsOps_destroyInodeCache(void);
extern struct inode* FhgfsOps_alloc_inode(struct super_block* sb);
extern void FhgfsOps_destroy_inode(struct inode* inode);

extern struct inode* __FhgfsOps_newInode(struct super_block* sb, struct kstat* kstat, dev_t dev,
   EntryInfo* entryInfo);
extern int __FhgfsOps_instantiateInode(struct dentry* dentry, EntryInfo* entryInfo,
   fhgfs_stat* fhgfsStat);
int __FhgfsOps_compareInodeID(struct inode* cachedInode, void* newInodeInfo);
int __FhgfsOps_initNewInodeDummy(struct inode* newInode, void* newInodeInfo);

extern int __FhgfsOps_refreshInode(App* app, struct inode* inode, fhgfs_stat* fhgfsStat);
extern int __FhgfsOps_revalidateMapping(App* app, struct inode* inode);
extern int __FhgfsOps_flushInodeFileCache(App* app, struct inode* inode);


// inliners
static inline void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, struct inode* outInode);
static inline void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat,
   struct inode* outInode);
static inline void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode);
static inline void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat, struct inode* inOutInode);


/**
 * This structure is passed to _compareInodeID().
 */
struct FhgfsInodeComparisonInfo
{
   ino_t inodeHash; // (=> inode::i_ino)
   const char* entryID;
};


/**
 * Note: acquires i_lock spinlock to protect i_size and i_blocks
 */
void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   spin_lock(&outInode->i_lock); // I _ L O C K

   __FhgfsOps_applyStatSizeToInode(kstat, outInode);

   spin_unlock(&outInode->i_lock); // I _ U N L O C K
}

/**
 * Note: Caller must hold i_lock.
 */
void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat, struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   __FhgfsOps_applyStatSizeToInode(kstat, outInode);
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 */
void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode)
{
   App* app = FhgfsOps_getApp(outInode->i_sb);
   Config* cfg = App_getConfig(app);

   // remote attribs (received from nodes)
   outInode->i_mode = kstat->mode;
   outInode->i_uid = kstat->uid;
   outInode->i_gid = kstat->gid;
   outInode->i_atime = kstat->atime;
   outInode->i_mtime = kstat->mtime;
   outInode->i_ctime = kstat->ctime;

   set_nlink(outInode, kstat->nlink);

   outInode->i_blkbits = Config_getTuneInodeBlockBits(cfg);

#ifdef KERNEL_HAS_IBLKSIZE
   outInode->i_blksize = Config_getTuneInodeBlockSize(cfg);
#endif // KERNEL_HAS_IBLKSIZE
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 * Note: Caller must hold i_lock.
 */
void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat, struct inode* inOutInode)
{
   struct super_block* sb = inOutInode->i_sb;
   App* app = FhgfsOps_getApp(sb);
   Config* cfg = App_getConfig(app);
   FhgfsInode* fhgfsInode = FHGFS_INODE(inOutInode);

   if( (Config_getTuneFileCacheTypeNum(cfg) == FILECACHETYPE_Paged) )
   {
      loff_t oldSize = i_size_read(inOutInode);

      /* We don't allow to decrease the file size in paged mode for write-opened files, as
       * this may/will cause data corruption (zeros/holes instead of real data */
      if (oldSize > kstat->size && FhgfsInode_getHasDirtyPages(fhgfsInode) )
         return;
   }

   i_size_write(inOutInode, kstat->size);
   inOutInode->i_blocks = (512-1 + kstat->size) >> 9;
      // note: i_blocks is in 512-byte units (see man stat(2) )
      // note: things with a positive size have at least one block with this
}

#endif /* FHGFSOPSINODE_H_ */
