#ifndef FHGFSOPS_IOCTL_H
#define FHGFSOPS_IOCTL_H

#include <asm/ioctl.h>

#define FHGFS_IOCTL_CFG_MAX_PATH 4096 // just an arbitrary value, has to be identical in user space

struct Fhgfs_ioctlMkFile;
typedef struct Fhgfs_ioctlMkFile Fhgfs_ioctlMkFile;


/* used to return the client config file name using an IOCTL */
struct Fhgfs_ioctlGetCfgFile
{
   // note: must be defined identically in user space; even a different size of IOCTL_CFG_MAX_PATH
   //       will make the IOCTL fail, as the size is encoded into the IOCTL by _IOR()
   char path[FHGFS_IOCTL_CFG_MAX_PATH];  /* must be pre-allocated in user space */
   int length;                            /* length of the pre-allocated char array */
};

/* Used to pass information about a new file to the kernel.
 *
 * note: must be defined identically in user space in order to have identical
 *       FHGFS_IOC_CREATE_FILE numbers in user and kernel space.
 */
struct Fhgfs_ioctlMkFile
{
      uint16_t ownerNodeID; // owner node of the parent dir

      const char* parentParentEntryID; // entryID of the parent of the parent (=> the grandparent entryID)
      int parentParentEntryIDLen;

      const char* parentEntryID; // entryID of the parent
      int parentEntryIDLen;

      const char* parentName;   // name of parent dir
      int parentNameLen;

      // file information
      const char* entryName; // file name we want to create
      int entryNameLen;
      int fileType; // see linux/fs.h or man 3 readdir, DT_UNKNOWN, DT_FIFO, ...

      char* symlinkTo;  // Only must be set for symlinks. The name a symlink is supposed to point to
      int symlinkToLen; // Length of the symlink name

      int mode; // mode (permission) of the new file

      // user ID and group only will be used, if the current user is root
      uid_t uid; // user ID
      gid_t gid; // group ID

      int numTargets;     // number of targets in prefTargets array (without final 0 element)
      uint16_t* prefTargets;  // array of preferred targets (additional last element must be 0)
      int prefTargetsLen; // raw byte length of prefTargets array (including final 0 element)
};

/*
 * clients IOCTL commands
 */
#define FHGFS_IOC_GETVERSION               _IOR('f', 3, long)
#ifdef FS_IOC_GETVERSION
   #define FHGFS_IOC_GETVERSION_OLD        FS_IOC_GETVERSION
#else 
   // old kernels didn't have it
   #define FHGFS_IOC_GETVERSION_OLD        _IOR('v', 1, long)
#endif

#ifdef CONFIG_COMPAT
   #define FHGFS_IOC32_GETVERSION           _IOR('f', 3, int)
   #ifdef FS_IOC32_GETVERSION
      #define FHGFS_IOC32_GETVERSION_OLD       FS_IOC32_GETVERSION
   #else
      // old kernels didn't have it
      #define FHGFS_IOC32_GETVERSION_OLD       _IOR('v', 1, int)
   #endif
#endif

/* return the config file used by the client */
#define FHGFS_IOC_GET_CFG_FILE             _IOR('f', 20, struct Fhgfs_ioctlGetCfgFile)

/* create files using an ioctl */
#define FHGFS_IOC_CREATE_FILE              _IOW('f', 21, struct Fhgfs_ioctlMkFile)

/* test if we are on FhGFS, outString will contain "FhGFS" if so */
#define FHGFS_IOC_TEST_IS_FHGFS            _IOR('f', 22, char[6])

/* return the runtime config file used by the client */
#define FHGFS_IOC_GET_RUNTIME_CFG_FILE     _IOR('f', 23, struct Fhgfs_ioctlGetCfgFile)

// will be returned to user space, NOTE: including \0 those are 8-byte and so optimal for memcmp
#define FHGFS_IOCTL_TEST_STRING           "_FhGFS_"


#endif // FHGFSOPS_IOCTL_H
