/*
 * fhgfs page cache methods
 *
 */

#ifndef FHGFSOPSPAGES_H_
#define FHGFSOPSPAGES_H_

#define FHGFS_MAX_PAGE_LIST_SIZE (8192) // we allow a maximum page list size and therefore right now
                                        // also IO size of 8192 pages, so 32MiB
                                        // NOTE: *MUST* be larger than INITIAL_FIND_PAGES

extern int FhgfsOps_readpagesVec(struct file* file, struct address_space* mapping,
   struct list_head* page_list, unsigned num_pages);
extern int FhgfsOps_writepagesVec(struct address_space* mapping, struct writeback_control* wbc);

static inline void FhgfsOpsPages_incInodeFileSizeOnPagedRead(struct inode* inode, loff_t offset,
   size_t readRes);
extern void __FhgfsOpsPages_incInodeFileSizeOnPagedRead(struct inode* inode, loff_t offset,
   size_t readRes);

/**
 * If the meta server has told us for some reason a wrong file-size (i_size) the caller would
 * wrongly discard data beyond i_size. So we are going to correct i_size here.
 *
 * TODO: This can be removed once we are sure the meta server *always* has the correct i_size.
 *
 * Note: This is the fast inline version, which should almost always only read i_size and then
 *       return. So mostly no need to increase the stack size.
 * */
void FhgfsOpsPages_incInodeFileSizeOnPagedRead(struct inode* inode, loff_t offset, size_t readRes)
{
   loff_t i_size = i_size_read(inode);

   if (unlikely(readRes && (offset + readRes > i_size) ) )
      __FhgfsOpsPages_incInodeFileSizeOnPagedRead(inode, offset, readRes);

}

#endif /* FHGFSOPSPAGES_H_ */
