#ifndef FHGFSOPSCOMMKIT_H_
#define FHGFSOPSCOMMKIT_H_

#include "FhgfsOpsCommKitCommon.h"

struct ReadfileState;
typedef struct ReadfileState ReadfileState;
struct WritefileState;
typedef struct WritefileState WritefileState;

extern void FhgfsOpsCommKit_readfileV2bCommunicate(App* app, ReadfileState* states,
   size_t numStates, PollSocketEx* pollSocks);
extern void FhgfsOpsCommKit_writefileV2bCommunicate(App* app, WritefileState* states,
   size_t numStates, PollSocketEx* pollSocks);


extern fhgfs_bool __FhgfsOpsCommKit_readfileV2bStagePREPARE(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageSEND(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageRECVHEADER(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageRECVDATA(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageSOCKETEXCEPTION(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageSOCKETINVALIDATE(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStageCLEANUP(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);

extern fhgfs_bool __FhgfsOpsCommKit_readfileV2bIsDataAvailable(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bPrepareRetry(RWfileStatesHelper* statesHelper,
   ReadfileState* currentState);
extern void __FhgfsOpsCommKit_readfileV2bStartRetry(RWfileStatesHelper* statesHelper,
   ReadfileState* states, size_t numStates);


extern fhgfs_bool __FhgfsOpsCommKit_writefileV2bStagePREPARE(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageSENDHEADER(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageSENDDATA(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageRECV(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageSOCKETEXCEPTION(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageSOCKETINVALIDATE(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStageCLEANUP(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);

extern fhgfs_bool __FhgfsOpsCommKit_writefileV2bIsDataAvailable(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bPrepareRetry(RWfileStatesHelper* statesHelper,
   WritefileState* currentState);
extern void __FhgfsOpsCommKit_writefileV2bStartRetry(RWfileStatesHelper* statesHelper,
   WritefileState* states, size_t numStates);



// inliners
static inline ReadfileState FhgfsOpsCommKit_assignReadfileState(char __user *buf,
   loff_t offset, size_t size, const char* fileHandleID, unsigned accessFlags, uint16_t targetID,
   char* msgBuf, fhgfs_bool firstWriteDoneForTarget);
static inline WritefileState FhgfsOpsCommKit_assignWritefileState(const char __user *buf,
   loff_t offset, size_t size, const char* fileHandleID, unsigned accessFlags, uint16_t targetID,
   char* msgBuf, fhgfs_bool firstWriteDoneForTarget);
static inline void FhgfsOpsCommKit_setWritefileStateMirror(uint16_t mirrorToTargetID,
   WritefileState* outState);
static inline void FhgfsOpsCommKit_setWritefileStateMirroredFrom(uint16_t mirroredFromTargetID,
   WritefileState* outState);
static inline void FhgfsOpsCommKit_setUserdataForQuota(WritefileState* this, unsigned userID,
   unsigned groupID);

static inline void __FhgfsOpsCommKit_readfileAddStateSockToPollSocks(
   RWfileStatesHelper* statesHelper, ReadfileState* currentState);
static inline void __FhgfsOpsCommKit_writefileAddStateSockToPollSocks(
   RWfileStatesHelper* statesHelper, WritefileState* currentState, short pollEvents);


struct ReadfileState
{
   // (set to Stage_PREPARE in the beginning, assigned by the state-creator)
   ReadfileStage stage; // the current stage of the individual communication process

   // data for spefic modes (will be assigned by the corresponding modes)
   size_t transmitted; // how much data has been transmitted already
   size_t toBeTransmitted; // how much data has to be transmitted
   size_t pollSocksIndex; // save the pollSocks-array index for the next round

   // data for all modes

   // (assigned by the state-creator)
   char __user *buf; // the data that shall be transferred
   loff_t offset; // target-node local offset
   size_t size; // size of the buf
   const char* fileHandleID;
   uint16_t targetID; // target ID
   char* msgBuf; // for serialization
   unsigned accessFlags;

   // (assigned by the preparation mode)
   Node* node; // target-node reference
   Socket* sock; // target-node connection

   // result data
   int64_t nodeResult; // the amount of data that has been read (or a negative fhgfs error code)
   int64_t expectedNodeResult; // the amount of data that we wanted to read

   fhgfs_bool firstWriteDoneForTarget; /* true if the first chunk was written to the storage target,
                                        it's needed for the session check*/
};

/**
 * Note: shares almost everything with ReadfileState, but deriving it from a common base-struct
 * would make the code (even more ;) unreadable
 */
struct WritefileState
{
   // (set to Stage_PREPARE in the beginning, assigned by the state-creator)
   WritefileStage stage; // the current stage of the individual communication process

   // data for spefic modes (will be assigned by the corresponding modes)
   size_t transmitted; // how much data has been transmitted already
   size_t toBeTransmitted; // how much data has to be transmitted
   size_t pollSocksIndex; // save the pollSocks-array index for the next round

   // data for all modes

   // (assigned by the state-creator)
   const char __user *buf; // the data that shall be transferred
   loff_t offset; // target-node local offset
   size_t size; // size of the buf
   const char* fileHandleID;
   uint16_t targetID; // target ID
   uint16_t mirrorToTargetID; // for server-side mirroring: data shall be forwarded to this mirror
   uint16_t mirroredFromTargetID; // for client-side mirroring: msg is a mirror of this targetID
   char* msgBuf; // for serialization
   unsigned accessFlags;

   unsigned userID;
   unsigned groupID;

   // (assigned by the preparation mode)
   Node* node; // target-node reference
   Socket* sock; // target-node connection
   WriteLocalFileRespMsg writeRespMsg; // response message

   // result data
   int64_t nodeResult; // the amount of data that has been written (or a negative fhgfs error code)
   int64_t expectedNodeResult; // the amount of data that we wanted to write

   fhgfs_bool firstWriteDoneForTarget; /* true if the first chunk was written to the storage target,
                                        it's needed for the session check*/
};


/**
 * Note: Initializes the expectedNodeResult attribute from the size argument
 */
ReadfileState FhgfsOpsCommKit_assignReadfileState(char __user *buf, loff_t offset, size_t size,
   const char* fileHandleID, unsigned accessFlags, uint16_t targetID, char* msgBuf,
   fhgfs_bool firstWriteDoneForTarget)
{
   ReadfileState state =
   {
      .stage = ReadfileStage_PREPARE,

      .buf = buf,
      .offset = offset,
      .size = size,
      .fileHandleID = fileHandleID,
      .targetID = targetID,
      .msgBuf = msgBuf,
      .accessFlags = accessFlags,

      .nodeResult = -FhgfsOpsErr_INTERNAL,
      .expectedNodeResult = size,

      .firstWriteDoneForTarget = firstWriteDoneForTarget,
   };

   return state;
}

/**
 * Note: Initializes the expectedNodeResult attribute from the size argument
 */
WritefileState FhgfsOpsCommKit_assignWritefileState(const char __user *buf, loff_t offset,
   size_t size, const char* fileHandleID, unsigned accessFlags, uint16_t targetID, char* msgBuf,
   fhgfs_bool firstWriteDoneForTarget)
{
   WritefileState state =
   {
      .stage = WritefileStage_PREPARE,

      .buf = buf,
      .offset = offset,
      .size = size,
      .fileHandleID = fileHandleID,
      .targetID = targetID,
      .mirrorToTargetID = 0, // set via FhgfsOpsCommKit_assignWritefileStateMirror()
      .mirroredFromTargetID = 0, // set via FhgfsOpsCommKit_assignWritefileStateMirroredFrom()
      .msgBuf = msgBuf,
      .accessFlags = accessFlags,

      .userID = 0,
      .groupID = 0,

      .nodeResult = -FhgfsOpsErr_INTERNAL,
      .expectedNodeResult = size,

      .firstWriteDoneForTarget = firstWriteDoneForTarget,
   };

   return state;
}

void FhgfsOpsCommKit_setWritefileStateMirror(uint16_t mirrorToTargetID, WritefileState* outState)
{
   outState->mirrorToTargetID = mirrorToTargetID;
}

void FhgfsOpsCommKit_setWritefileStateMirroredFrom(uint16_t mirroredFromTargetID,
   WritefileState* outState)
{
   outState->mirroredFromTargetID = mirroredFromTargetID;
}

void __FhgfsOpsCommKit_readfileAddStateSockToPollSocks(
   RWfileStatesHelper* statesHelper, ReadfileState* currentState)
{
   (statesHelper->pollSocks)[statesHelper->numPollSocks] =
      __FhgfsOpsCommKitCommon_assignPollSocket(currentState->sock, FHGFS_POLLIN);
   currentState->pollSocksIndex = statesHelper->numPollSocks;
   statesHelper->numPollSocks++;
}

/**
 * @param pollEvents the events to poll for (e.g. FHGFS_POLLIN or FHGFS_POLLOUT)
 */
void __FhgfsOpsCommKit_writefileAddStateSockToPollSocks(
   RWfileStatesHelper* statesHelper, WritefileState* currentState, short pollEvents)
{
   (statesHelper->pollSocks)[statesHelper->numPollSocks] =
      __FhgfsOpsCommKitCommon_assignPollSocket(currentState->sock, pollEvents);
   currentState->pollSocksIndex = statesHelper->numPollSocks;
   statesHelper->numPollSocks++;
}

void FhgfsOpsCommKit_setUserdataForQuota(WritefileState* this, unsigned userID,
   unsigned groupID)
{
   this->userID = userID;
   this->groupID = groupID;
}

#endif /*FHGFSOPSCOMMKIT_H_*/
