#ifndef FHGFSOPSCOMMKITVEC_H_
#define FHGFSOPSCOMMKITVEC_H_

#include "FhgfsOpsCommKitCommon.h"
#include <toolkit/FhgfsPage.h>

struct FileStateVec;
typedef struct FileStateVec FileStateVec;

extern void FhgfsOpsCommKitVec_writefileCommunicate(App* app, FileStateVec* states,
   size_t numStates, PollSocketEx* pollSocks);
extern void FhgfsOpsCommKitVec_readfileCommunicate(App* app, FileStateVec* states,
   size_t numStates, PollSocketEx* pollSocks);

static inline FileStateVec FhgfsOpsCommKitVec_assignRWfileState(FhgfsPageVec* pageVec,
   unsigned pageIdx, unsigned numPages, loff_t offset,  const char* fileHandleID,
   unsigned accessFlags, uint16_t targetID, char* msgBuf, int stateIdx,
   fhgfs_bool firstWriteDoneForTarget);
static inline void FhgfsOpsCommKitVec_setRWfileStateMirror(uint16_t mirrorTargetID,
   FileStateVec* outState);
static inline void FhgfsOpsCommKitVec_setUserdataForQuota(FileStateVec* state, unsigned userID,
   unsigned groupID);

#ifdef FhgfsOpsCommKitVec // local (file/class-like) declarations

static void __FhgfsOpsCommKitVec_addStateSockToPollSocks(
   RWfileStatesHelper* statesHelper, FileStateVec* curState, short pollEvents);


static fhgfs_bool __FhgfsOpsCommKitVec_readfileStagePREPARE(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStageSEND(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStageRECVDATA(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStageSOCKETEXCEPTION(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStageSOCKETINVALIDATE(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStageCLEANUP(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfilePrepareRetry(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static fhgfs_bool __FhgfsOpsCommKitVec_readFileIsDataAvailable(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_readfileStartRetry(RWfileStatesHelper* statesHelper,
   FileStateVec* states, size_t numStates);


static fhgfs_bool __FhgfsOpsCommKitVec_writefileStagePREPARE(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageSENDHEADER(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageSENDDATA(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageRECV(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageSOCKETEXCEPTION(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageSOCKETINVALIDATE(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStageCLEANUP(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefilePrepareRetry(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);
static void __FhgfsOpsCommKitVec_writefileStartRetry(RWfileStatesHelper* statesHelper,
   FileStateVec* states, size_t numStates);
static fhgfs_bool __FhgfsOpsCommKitVec_writeFileIsDataAvailable(RWfileStatesHelper* statesHelper,
   FileStateVec* curState);

#endif // local (file/class-like) declarations


struct FileStateVec
{
   // Depending if we are going to read or write, the corresponding struct will be used
   union
   {
         // NOTE: The *initial* stage for read and write MUST be RW_FILE_STAGE_PREPARE
         struct readState
         {
               // (set to Stage_PREPARE in the beginning, assigned by the state-creator)
               ReadfileStage stage; // the current stage of the individual communication process

               size_t sendDataLength; // how many data the server is going to send
               size_t receivedLength; // how many data we already got
         } read ;

         struct writeState
         {
               WritefileStage stage;
               WriteLocalFileRespMsg respMsg; // response message

               unsigned userID;
               unsigned groupID;
         } write;
   } type;

   size_t hdrLen; // size (length) of the header message

   /* We send data to the network layer until network buffer is full. Once the buffer is free we
    * can continue to send data, but of course we need to store information where we stopped last
    * time.
    */
   int contLastPage;   // last processed page within the vector
   int contPageOffset; // offset (already processed data) within the last page
   size_t contPollSocksIndex; // save the pollSocks-array index for the next round

   // data for all modes

   // (assigned by the state-creator)
   FhgfsPageVec pageVec;
   loff_t offset; // target-node local offset
   size_t dataSize;   // sum of all pageVec->page->length elements


   const char* fileHandleID;
   uint16_t targetID; // target ID
   uint16_t mirrorTargetID; // the data in this msg shall be forwarded to this mirror
   char* msgBuf; // for serialization
   unsigned accessFlags;
   fhgfs_bool firstWriteDoneForTarget; /* true if the first chunk was written to the storage target,
                                        it's needed for the session check*/

   // (assigned by the preparation mode)
   Node* node; // target-node reference
   Socket* sock; // target-node connection

   // result information
   int64_t nodeResult; /* the amount of data that have been read/written on the server side
                        * (or a negative fhgfs error code) */

#ifdef FHGFS_DEBUG
   int stateIdx; // just the n-th state for debug output
#endif
};

/**
 * Note:  Initializes the expectedNodeResult attribute from the size argument
 * Note2: Includeded directly in the header to inline it
 * Note3: *important*: Caller still MUST initialize type.{read/write}.stage = RW_FILE_STAGE_PREPARE
 */
static inline FileStateVec FhgfsOpsCommKitVec_assignRWfileState(FhgfsPageVec* pageVec,
   unsigned pageIdx, unsigned numPages, loff_t offset,  const char* fileHandleID,
   unsigned accessFlags, uint16_t targetID, char* msgBuf, int stateIdx,
   fhgfs_bool firstWriteDoneForTarget)
{
   /* we just pageVec->pages[] for all states, but each state with a different pageIdx offset
    * and numPages as vector size for their own .pageVec array */
   FhgfsPage* pageArray = &pageVec->pages[pageIdx];
   FhgfsPage* lastPage = &pageArray[numPages - 1];

   FileStateVec state =
   {
      .pageVec = {
         .size  = numPages,
         .pages = pageArray,
       },

      .offset             = offset,
      .fileHandleID       = fileHandleID,
      .targetID           = targetID,
      .mirrorTargetID     = 0, // set via FhgfsOpsCommKitVec_setRWfileStateMirror
      .msgBuf             = msgBuf,
      .accessFlags        = accessFlags,

      .firstWriteDoneForTarget   = firstWriteDoneForTarget,

      .nodeResult         = -FhgfsOpsErr_INTERNAL,


      // last page might not be fully used and so might a smaller size
      .dataSize = (numPages - 1) * PAGE_SIZE + lastPage->length,

      .contLastPage       = 0,
      .contPageOffset     = 0,

#ifdef FHGFS_DEBUG
      .stateIdx           = stateIdx,
#endif

   };

   return state;
}

void FhgfsOpsCommKitVec_setRWfileStateMirror(uint16_t mirrorTargetID, FileStateVec* outState)
{
   outState->mirrorTargetID = mirrorTargetID;
}

void FhgfsOpsCommKitVec_setUserdataForQuota(FileStateVec* state, unsigned userID,
   unsigned groupID)
{
   state->type.write.userID = userID;
   state->type.write.groupID = groupID;
}


#endif /*FHGFSOPSCOMMKITVEC_H_*/
