#ifndef FHGFSOPSREMOTING_H_
#define FHGFSOPSREMOTING_H_

#include <common/Common.h>
#include <filesystem/FsObjectInfo.h>
#include <filesystem/FsDirInfo.h>
#include <filesystem/FsFileInfo.h>
#include <common/storage/StorageDefinitions.h>
#include <common/toolkit/MetadataTk.h>
#include <common/storage/StorageErrors.h>
#include <net/filesystem/RemotingIOInfo.h>
#include <toolkit/FhgfsPage.h>


enum Fhgfs_RWType;
typedef enum Fhgfs_RWType Fhgfs_RWType;


struct StripePattern; // forward declaration


extern FhgfsOpsErr FhgfsOpsRemoting_listdirFromOffset(const EntryInfo* entryInfo,
   FsDirInfo* dirInfo, unsigned maxOutNames);
extern FhgfsOpsErr FhgfsOpsRemoting_statRoot(App* app, fhgfs_stat* outFhgfsStat);
extern FhgfsOpsErr FhgfsOpsRemoting_statDirect(App* app, const EntryInfo* entryInfo,
   fhgfs_stat* outFhgfsStat);
extern FhgfsOpsErr FhgfsOpsRemoting_setAttr(App* app, const EntryInfo* entryInfo,
   SettableFileAttribs* fhgfsAttr, int validAttribs);
extern FhgfsOpsErr FhgfsOpsRemoting_mkdir(App* app, const EntryInfo* parentInfo,
   struct CreateInfo* createnfo, EntryInfo* outEntryInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_rmdir(App* app, const EntryInfo* parentInfo,
   const char* entryName);
extern FhgfsOpsErr FhgfsOpsRemoting_mkfile(App* app, const EntryInfo* parentInfo,
   struct CreateInfo* createInfo, EntryInfo* outEntryInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_unlinkfile(App* app, const EntryInfo* parentInfo,
   const char* entryName);
extern FhgfsOpsErr FhgfsOpsRemoting_openfile(const EntryInfo* entryInfo, RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_closefile(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_closefileEx(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo, fhgfs_bool allowRetries);
extern FhgfsOpsErr FhgfsOpsRemoting_flockEntry(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo, int64_t clientFD, int ownerPID, int lockTypeFlags);
extern FhgfsOpsErr FhgfsOpsRemoting_flockEntryEx(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo, int64_t clientFD, int ownerPID, int lockTypeFlags,
   fhgfs_bool allowRetries);
extern FhgfsOpsErr FhgfsOpsRemoting_flockRange(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo, int ownerPID, int lockTypeFlags, uint64_t start, uint64_t end);
extern FhgfsOpsErr FhgfsOpsRemoting_flockRangeEx(const EntryInfo* entryInfo,
   RemotingIOInfo* ioInfo, int ownerPID, int lockTypeFlags, uint64_t start, uint64_t end,
   fhgfs_bool allowRetries);
extern ssize_t FhgfsOpsRemoting_writefile(const char __user *buf, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsRemoting_RWfileVec(FhgfsPageVec *pageVec, loff_t offset,
   RemotingIOInfo* ioInfo, Fhgfs_RWType rwType);
extern ssize_t FhgfsOpsRemoting_readfile(char __user *buf, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_rename(App* app, const char* oldName, unsigned oldLen,
   DirEntryType entryType, EntryInfo* fromDirInfo, const char* newName, unsigned newLen,
   EntryInfo* toDirInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_truncfile(App* app, const EntryInfo* entryInfo, loff_t size);
extern FhgfsOpsErr FhgfsOpsRemoting_fsyncfile(RemotingIOInfo* ioInfo, fhgfs_bool forceRemoteFlush,
   fhgfs_bool checkSession, fhgfs_bool doSyncOnClose);
extern FhgfsOpsErr FhgfsOpsRemoting_statStoragePath(App* app, int64_t* outSizeTotal,
   int64_t* outSizeFree);

extern FhgfsOpsErr FhgfsOpsRemoting_validateAndStatSubentry(App* app,
   const EntryInfo* parentEntryInfo, const char* entryName, const EntryInfo* entryInfo,
   FhgfsOpsErr* outStatRes, fhgfs_stat* outFhgfsStat);
extern FhgfsOpsErr FhgfsOpsRemoting_lookupAndStatSubentry(App* app,
   const EntryInfo* parentEntryInfo, const char* entryName, EntryInfo* outEntryInfo,
   FhgfsOpsErr* outStatRes, fhgfs_stat* outFhgfsStat);

extern FhgfsOpsErr FhgfsOpsRemoting_lookupCreateStatFile(App* app,
   const LookupCreateStatInfoIn* inInfo, LookupCreateStatInfoOut* outInfo);

extern FhgfsOpsErr FhgfsOpsRemoting_hardlink(App* app, const char* fromName, unsigned fromLen,
   EntryInfo* fromInfo, EntryInfo* fromDirInfo, const char* toName, unsigned toLen,
   EntryInfo* toDirInfo);

extern FhgfsOpsErr FhgfsOpsRemoting_refreshEntry(App* app, const EntryInfo* entryInfo);


#ifdef LOG_DEBUG_MESSAGES
extern void __FhgfsOpsRemoting_logDebugIOCall(const char* logContext, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
#else
#define __FhgfsOpsRemoting_logDebugIOCall(logContext, size, offset, ioInfo)
#endif // LOG_DEBUG_MESSAGES


enum Fhgfs_RWType
{
   FHGFS_RWTYPE_READ = 0,  // we are going to read
   FHGFS_RWTYPE_WRITE  // we are going to write
};


#endif /*FHGFSOPSREMOTING_H_*/
