#ifndef REMOTINGIOINFO_H_
#define REMOTINGIOINFO_H_

#include <common/Common.h>
#include <common/storage/striping/StripePattern.h>
#include <toolkit/BitStore.h>
#include <app/App.h>


struct RemotingIOInfo;
typedef struct RemotingIOInfo RemotingIOInfo;


extern unsigned RemotingIOInfo_getNumPagesPerStripe(RemotingIOInfo* ioInfo);

// inliners
static inline void RemotingIOInfo_initOpen(App* app, unsigned accessFlags,
   AtomicInt* pMaxUsedTargetIndex, RemotingIOInfo* outIOInfo);
static inline void RemotingIOInfo_freeVals(RemotingIOInfo* outIOInfo);



struct RemotingIOInfo
{
      App* app;

      const char* fileHandleID;
      struct StripePattern* pattern;
      unsigned accessFlags; // OPENFILE_ACCESS_... flags

      // pointer to fhgfsInode->fileHandles[handleType].maxUsedTargetIndex
      AtomicInt* maxUsedTargetIndex;
      BitStore* firstWriteDone;

      unsigned userID;     // only used in storage server write message
      unsigned groupID;    // only used in storage server write message
};


/**
 * Prepares a RemotingIOInfo for Remoting_openfile().
 *
 * Note: Be careful with this. This is only useful in very special cases (e.g. stateless file IO,
 * when you called FhgfsOpsRemoting_openfile() directly). You will typically rather use the
 * FhgfsInode_referenceHandle() & co routines.
 *
 * @param accessFlags OPENFILE_ACCESS_... flags
 * @param maxUsedTargetIndex may be NULL for open, but in case you will use the ioInfo also for
 * IO, you will probably want to set it.
 */
void RemotingIOInfo_initOpen(App* app, unsigned accessFlags, AtomicInt* maxUsedTargetIndex,
   RemotingIOInfo* outIOInfo)
{
   outIOInfo->app = app;
   outIOInfo->fileHandleID = NULL;
   outIOInfo->pattern = NULL;
   outIOInfo->accessFlags = accessFlags;
   outIOInfo->maxUsedTargetIndex = maxUsedTargetIndex;
   outIOInfo->firstWriteDone = NULL;
   outIOInfo->userID = 0;
   outIOInfo->groupID = 0;
}

/**
 * Note: Be careful with this. This is only useful in very special cases (e.g. stateless file IO,
 * when you called Remoting_openfile() directly). You will typically rather use the
 * FhgfsInode_referenceHandle() & Co routines.
 */
void RemotingIOInfo_freeVals(RemotingIOInfo* outIOInfo)
{
   SAFE_DESTRUCT(outIOInfo->pattern, StripePattern_virtualDestruct);
   SAFE_KFREE(outIOInfo->fileHandleID);
}


#endif /* REMOTINGIOINFO_H_ */
