#include "Program.h"
#include <app/App.h>
#include <common/toolkit/SocketTk.h>
#include <filesystem/FhgfsOpsSuper.h>
#include <filesystem/FhgfsOpsInode.h>
#include <filesystem/ProcFs.h>


Program program;


int Program_main(void)
{
//   int appRes;
   fhgfs_bool assumptionsRes;
   fhgfs_bool sockRes;
   fhgfs_bool cacheRes;

   assumptionsRes = os_checkCompileTimeAssumptions();
   if(!assumptionsRes)
   {
      printk_fhgfs(KERN_WARNING, "Compile-time assumptions were wrong. "
         "Please contact support.");
      return APPCODE_PROGRAM_ERROR;
   }

   sockRes = SocketTk_initOnce();
   if(!sockRes)
   {
      printk_fhgfs(KERN_WARNING, "SocketTk initialization failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   cacheRes = FhgfsOps_initInodeCache();
   if(!cacheRes)
   {
      printk_fhgfs(KERN_WARNING, "Inode cache initialization failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   program.registerRes = FhgfsOps_registerFilesystem();
   if(program.registerRes)
   {
      printk_fhgfs(KERN_WARNING, "File system registration failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   printk_fhgfs(KERN_INFO, "File system registered. Type: fhgfs. Version: %s\n",
      App_getVersionStr() );

   ProcFs_createGeneralDir();

   return 0;
}

void Program_exit(void)
{
   ProcFs_removeGeneralDir();

   if(!program.registerRes)
   {
      int unregisterRes;

      unregisterRes = FhgfsOps_unregisterFilesystem();
      if(unregisterRes)
         printk_fhgfs(KERN_WARNING, "File system deregistration failed\n");
   }

   FhgfsOps_destroyInodeCache();

   SocketTk_uninitOnce();

   printk_fhgfs(KERN_INFO, "FhGFS Client unloaded.\n");
}
