#include <toolkit/SignalTk.h>
#include <common/Common.h>

#include <linux/signal.h>


// Note: SIGKILL will always be allowed by SignalTk_blockSignals()
#define SIGNALTK_ALLOWED_SIGNALS \
   (sigmask(SIGTERM) | sigmask(SIGQUIT) | sigmask(SIGHUP) )


/**
 * Manipulates the signal mask of a thread by blocking signals that would have a bad influence
 * during remoting.
 *
 * Note: Do not forget to call __SignalTk_restoreSignals() later.
 *
 * @param allowSigInt usually fhgfs_true (fhgfs_false probably only useful for logging)
 * @param outOldSignalSet the original signal set
 */
void SignalTk_blockSignals(fhgfs_bool allowSigInt, fhgfs_sigset_t* outOldSignalSet)
{
   static unsigned long allowedSigs =
      sigmask(SIGKILL) | SIGNALTK_ALLOWED_SIGNALS;
   static unsigned long allowedSigsPlusInt =
      sigmask(SIGKILL) | SIGNALTK_ALLOWED_SIGNALS | sigmask(SIGINT);

   unsigned long currentAllowedSigs =
      allowSigInt ? allowedSigsPlusInt : allowedSigs;
   sigset_t newSigmask;
   sigset_t oldSigset;


   // invert signal mask (we actually need the blocked signals)
   siginitsetinv(&newSigmask, currentAllowedSigs);

   // block signals (that are not contained in allowedSigs)
   sigprocmask(SIG_BLOCK, &newSigmask, &oldSigset);

   // note: we can only do it this way because we verify the equal sizes of these types in
   // os_checkCompileTimeAssumptions() !
   memcpy(outOldSignalSet, &oldSigset, sizeof(oldSigset) );
}

void SignalTk_restoreSignals(fhgfs_sigset_t* oldSignalSet)
{
   sigset_t sigset;

   // note: we can only do it this way because we verify the equal sizes of these types in
   // os_checkCompileTimeAssumptions() !
   memcpy(&sigset, oldSignalSet, sizeof(sigset) );

   sigprocmask(SIG_SETMASK, &sigset, NULL);
}
